var max_geo_results = 5;

function createMarker(point, desc)
{
	var marker = new GMarker(point);

	// Note: Without wrapping into a function, listeners are added to the same objects!
	GEvent.addListener(marker, "click", function() {
		marker.openInfoWindowHtml(desc);
	});

	return marker;
}

function queryGoogleGeo(map, query, zoom)
{
	var geocoder = new GClientGeocoder();

	geocoder.getLocations(query,
		function generateMarkersFromGoogleGeoResult(response)
		{
			// Was not able to locate any data:
			if (response == null || response.Status.code != 200) {
				alert("Sorry, we were unable to locate " + query + " address");
				return;
			}

			var places = response.Placemark;

			for (var i = 0; i < places.length && i < max_geo_results; i++) {
				var place = places[i];
				var point = new GLatLng(place.Point.coordinates[1], place.Point.coordinates[0]);

				if (i == 0) {
					map.setCenter(point, zoom);
				}

				map.addOverlay(createMarker(point, '<div class="gmapsmarker"><strong>' + place.address + '</strong><br/>'
					+ place.AddressDetails.Country.CountryNameCode
				));
			}
		});
}

/**
 * Initialisation function. Creates Gmap objects and loads Geo information.
 */
function loadMaps()
{
	var headNode = document.getElementsByTagName("head").item(0);
	var divNodes = document.body.getElementsByTagName('div');

	for (var i = 0; i < divNodes.length; i++) {
		if (divNodes[i].className.match(/\bgmapsint\b/)) {
			var attrs = divNodes[i].attributes;

			// Create a map:
			var map = new GMap2(divNodes[i]);
			map.setCenter(new GLatLng(34, 0), 1); // default point

			// left-top navigator and zoomer
			if (attrs.size.value == 'small')
				map.addControl(new GSmallMapControl());
			else
			if (attrs.size.value == 'large')
				map.addControl(new GLargeMapControl());

			// right-top map type switch buttons
			if (attrs.control.value == 'hierarchical')
				map.addControl(new GHierarchicalMapTypeControl());
			else
			if (attrs.control.value == 'default')
				map.addControl(new GMapTypeControl());

			// mini-map in the bottom-right corner
			if (attrs.overviewmap.value == 'true') {
				var overviewMap = new GOverviewMapControl();
				map.addControl(overviewMap);
				overviewMap.hide();
			}

			map.enableScrollWheelZoom();

			var query = attrs.query.value;
			var zoom = parseInt(attrs.zoom.value);

			//queryGeonames(map, query, zoom);
			queryGoogleGeo(map, query, zoom);
		}
	}
}

// A special Wiki-wide function, defined in lib/scripts/events.js:
addInitEvent(loadMaps);
