<?php
/**
 * Plugin google_map: Generates external or internal link to Google Maps.
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Dmitry Katsubo <dma_k@mail.ru>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_google_maps extends DokuWiki_Syntax_Plugin {

	var $config;
  
	function syntax_plugin_google_maps() {
		$this->config = confToHash(DOKU_PLUGIN.'google_maps/syntax.conf');
	}

	function getInfo() {
		return array(
			'author1'	=> 'Dmitry Katsubo',
			'email1'	=> 'dma_k@mail.ru',
			'date'		=> '2008-04-12',
			'name'		=> 'Google Maps Plugin',
			'desc'		=> 'Adds a Google Maps frame 
				 syntax: {{googlemaps>address[zoom=16,size=small,control=hierarchical,overviewmap=true,width=800,height=600,type=internal]|alternate text}}',
			'url'    => 'http://centurion.dynalias.com/wiki/plugin/google_maps',
		);
	}

    function getType()  { return 'substition'; }
	function getSort()  { return 159; }

	function connectTo($mode) {
		$this->Lexer->addSpecialPattern('\{\{googlemaps>[^}]+\}\}',$mode,'plugin_google_maps'); 
	}

	function handle($match, $state, $pos, &$handler) {
		$matches = array();
		if (!preg_match('/{{googlemaps>([^\[|]+)(?:\[(.+)\])?(?:\|(.+))?}}/', $match, $matches)) {
			return array('error', $this->getLang("gmaps_Bad_iFrame"));  // this is an error
		}

		$options = array();

		if (isset($matches[2])) {
			preg_replace("/\s+/", '', $matches[2]);
			$entries = explode(',', $matches[2]);

			foreach ($entries as $entry) {
				$key_value = explode('=', $entry);

				$options[$key_value[0]] = $key_value[1];
			}
		}

		$query = html_entity_decode(trim($matches[1]));
		$alt = isset($matches[3]) ? html_entity_decode(trim($matches[3])) : $matches[1];

		return array('wiki', htmlspecialchars($query), $alt, &$options); 
	}

	function getConfigValue($options, $option_name, $config_prefix = null)
	{
		// Also escape HTML to protect the page:
		return(htmlspecialchars(
			isset($options[$option_name]) ?
			$options[$option_name] :
			$this->config[$config_prefix . $option_name]
		));
	}

	function render($mode, &$renderer, $data) {
		list($style, $query, $alt, $options) = $data;

		if($mode == 'xhtml') {
			// Two styles: wiki and error
			switch($style) {
			case 'wiki':
				if ($options['type'] == 'internal') {
					// Injection of this script causes FF to hang. so we have to generate it for each map:
					$renderer->doc .= "\n<script type='text/javascript' src='http://maps.google.com/maps?file=api&v=2.x&key=" . $this->config['google_api_key'] . "'></script>";

					// Default values:
					$size			= $this->getConfigValue($options, 'size');
					$width  		= $this->getConfigValue($options, 'width', $size . '_') . "px";
					$height 		= $this->getConfigValue($options, 'height', $size . '_') . "px";

					// Internal div:
					$renderer->doc .= "\n<div class='gmapsint' style='width: $width; height: $height' query='$query' ";

					// Copy values into attributes:
					foreach (array('size', 'control', 'overviewmap', 'zoom') as $attr_name) {
						$renderer->doc .= " $attr_name='" . $this->getConfigValue($options, $attr_name) . "'";
					}

					$renderer->doc .= "></div>"; // Important to leave one hanging node inside <div>, otherwise maps start overlappig
				}
				else {
					// Concat params:
					$params = '&';
					// If not defined, Google Maps engine will automatically select the best zoom:
					if ($options['zoom']) { $params .= "z=" . $options['zoom']; }

					// Query is already escaped, params are taken from options:
					$url = "http://maps.google.com/maps?q=$query$params";

					// External link:
					$renderer->doc .= "<a href='$url' class='gmapsext'>$alt</a>";
				}

				break;

			case 'error':
				$renderer->doc .= "<div class='error'>$query</div>";
				break;

			default:
				$renderer->doc .= "<div class='error'>" . $this->getLang('gmaps_Invalid_mode') . "</div>";
				break;
			}

			return true;
		}

		return false;
	}
}
?>
